
import 'package:equatable/equatable.dart';

sealed class CurtainState  extends Equatable {
  const CurtainState();

  @override
  List<Object> get props => [];
}

final class CurtainInitial extends CurtainState {}

class CurtainStatusLoading extends CurtainState {}

class CurtainStatusLoaded extends CurtainState {
  final bool status;

  const CurtainStatusLoaded(this.status);

  @override
  List<Object> get props => [status];
}

class CurtainError extends CurtainState {
  final String message;

  const CurtainError(this.message);

  @override
  List<Object> get props => [message];
}

class CurtainControlError extends CurtainState {
  final String message;

  const CurtainControlError(this.message);

  @override
  List<Object> get props => [message];
}
